@@ -74,7 +74,7 @@ class CidClient:
         """
         Gets the crafts available and the elements that it is composed of.
         """
-        return self._user_info["instances_v0_1"]
+        return self._user_info["user"]["instances_v0_1"]
 
     def fetch_craft(self, craft_id):
         """
@@ -96,7 +96,7 @@ class CidClient:
         # TODO might want to keep these separated? or at a minimum store a mapping from node guid to crate guid that it belongs to
         aggregate_crate = []
         for crate in craft["instance_nodes"]:
-            aggregate_crate += self._fetch_instance_crate(crate)["payload"]
+            aggregate_crate += self._fetch_instance_crate(crate)["payload"]["crates"]
 
         # Pull down and create the tree associated with the craft
         the_tree = self._fetch_instance_tree(craft["tree"])
