@@ -2,7 +2,7 @@
 import datetime
 import json
 import os
-import fasteners
+import uuid
 
 import pyrebase
 
@@ -54,7 +54,6 @@ class Authentication(object):
         :param skip_refresh_on_init: By default we refresh the token if needed upon object construction
         """
         self._auth_path = auth_path if auth_path else self.kAuthPath
-        self._auth_lock = fasteners.ReaderWriterLock() # Lock for threading within this process. Interprocess is not supported at this time.
         self._verbose = verbose
 
         self.printifv(f"Loading authentication from {os.path.abspath(self._auth_path)}")
@@ -144,25 +143,31 @@ class Authentication(object):
 
     def _write_file(self, id_token, refresh_token, maybe_id_token_expire_time):
         """
-        This creates or overwrites
+        This creates or overwrites the authentication file. 
+
+        Note: To mitigate multiple threads/processes interacting with the file, 
+        we write to a unique temporary file and swap that into the true path.
+        This relies on the OS/python's lib atomicity using os.replace()
         """
-        # Grab a write lock to the file
-        with self._auth_lock.write_lock():
-            # Then try to remove and refresh the file
-            try:
-                os.remove(self._auth_path)
-            except:
-                pass
-            with open(self._auth_path, "w") as auth_file:
-                json.dump(
-                    {
-                        "id_token": id_token,
-                        "refresh_token": refresh_token,
-                        "maybe_id_token_expire_time": maybe_id_token_expire_time,
-                    },
-                    auth_file,
-                )
-        
+        # Write the contents to a unique file in the same folder
+        folder_path = os.path.split(self._auth_path)[0]
+        temp_file_path = os.path.join(folder_path, f"{uuid.uuid4()}.tmp")
+        with open(temp_file_path, 'w') as temp_auth_file:
+            json.dump(
+                {
+                    "id_token": id_token,
+                    "refresh_token": refresh_token,
+                    "maybe_id_token_expire_time": maybe_id_token_expire_time,
+                },
+                temp_auth_file,
+            )
+
+        # Now, let atomically swap in our temporary updated auth file
+        try:
+            os.replace(temp_file_path, self._auth_path)
+        except Exception as e:
+            print(f"Failed to refresh authentication file, {e}")
+
 
     def _load_file(self):
         """
@@ -175,18 +180,15 @@ class Authentication(object):
                 f"File does not exist: {self._auth_path}"
             )
 
-        # Grab a read lock to the file
-        with self._auth_lock.read_lock():
-            with open(self._auth_path) as auth_file:
-                try:
-                    keys = json.load(auth_file)
-                    id_token = keys["id_token"]
-                    refresh_token = keys["refresh_token"]
-                    maybe_id_token_expire_time = keys["maybe_id_token_expire_time"]
-
-                    return id_token, refresh_token, maybe_id_token_expire_time
-                except:
-                    raise FailedLoadingAuthFileException(
-                        f"Corrupt: could not read required keys: {self._auth_path}"
-                    )
+        with open(self._auth_path) as auth_file:
+            try:
+                keys = json.load(auth_file)
+                id_token = keys["id_token"]
+                refresh_token = keys["refresh_token"]
+                maybe_id_token_expire_time = keys["maybe_id_token_expire_time"]
+                return id_token, refresh_token, maybe_id_token_expire_time
+            except:
+                raise FailedLoadingAuthFileException(
+                    f"Corrupt: could not read required keys: {self._auth_path}"
+                )
 
